@@ -30,6 +30,7 @@ class KBookmark;
 class KConfigDialog;
 class KXMLGUIClient;
 class KUrl;
+class DocumentItem;
 
 namespace Okular {
 
@@ -43,6 +44,9 @@ class DocumentViewport;
 class EmbeddedFile;
 class ExportFormat;
 class FontInfo;
+class FormFieldText;
+class FormFieldButton;
+class FormFieldChoice;
 class Generator;
 class Action;
 class MovieAction;
@@ -181,7 +185,7 @@ class OKULAR_EXPORT Document : public QObject
          * Sets the list of visible page rectangles.
          * @see VisiblePageRect
          */
-        void setVisiblePageRects( const QVector< VisiblePageRect * > & visiblePageRects, int excludeId = -1 );
+        void setVisiblePageRects( const QVector< VisiblePageRect * > & visiblePageRects, DocumentObserver *excludeObserver = 0 );
 
         /**
          * Returns the list of visible page rectangles.
@@ -295,18 +299,18 @@ class OKULAR_EXPORT Document : public QObject
         /**
          * Sets the current document viewport to the given @p page.
          *
-         * @param excludeId The observer ids which shouldn't be effected by this change.
+         * @param excludeObserver The observer ids which shouldn't be effected by this change.
          * @param smoothMove Whether the move shall be animated smoothly.
          */
-        void setViewportPage( int page, int excludeId = -1, bool smoothMove = false );
+        void setViewportPage( int page, DocumentObserver *excludeObserver = 0, bool smoothMove = false );
 
         /**
          * Sets the current document viewport to the given @p viewport.
          *
-         * @param excludeId The observer ids which shouldn't be effected by this change.
+         * @param excludeObserver The observer which shouldn't be effected by this change.
          * @param smoothMove Whether the move shall be animated smoothly.
          */
-        void setViewport( const DocumentViewport &viewport, int excludeId = -1, bool smoothMove = false );
+        void setViewport( const DocumentViewport &viewport, DocumentObserver *excludeObserver = 0, bool smoothMove = false );
 
         /**
          * Sets the current document viewport to the next viewport in the
@@ -334,10 +338,8 @@ class OKULAR_EXPORT Document : public QObject
 
         /**
          * Sets the zoom for the current document.
-         *
-         * @param excludeId The observer ids which shouldn't be effected by this change.
          */
-        void setZoom( int factor, int excludeId = -1 );
+        void setZoom( int factor, DocumentObserver *excludeObserver = 0 );
 
         /**
          * Describes the possible options for the pixmap requests.
@@ -383,23 +385,46 @@ class OKULAR_EXPORT Document : public QObject
         bool canModifyPageAnnotation( const Annotation * annotation ) const;
 
         /**
-         * Modifies the given @p annotation on the given @p page.
+         *  Prepares to modify the properties of the given @p annotation.
+         *  Must be called before the annotation's properties are modified
          *
-         * Same as calling modifyPageAnnotation(int,Annotation*,bool) with
-         * appearanceChanged = true
+         * @since 0.17 (KDE 4.11)
          */
-        void modifyPageAnnotation( int page, Annotation *annotation );
+        void prepareToModifyAnnotationProperties( Annotation * annotation );
 
         /**
          * Modifies the given @p annotation on the given @p page.
+         * Must be preceded by a call to prepareToModifyAnnotationProperties before
+         * the annotation's properties are modified
          *
-         * The caller can set @p appearanceChanged to false if it didn't change
-         * the annotation appearance (because it only changed non-visible data
-         * such as timestamps or author name).
-         *
-         * @since 0.15 (KDE 4.9)
+         * @since 0.17 (KDE 4.11)
          */
-        void modifyPageAnnotation( int page, Annotation *annotation, bool appearanceChanged );
+        void modifyPageAnnotationProperties( int page, Annotation * annotation );
+
+        /**
+         * Translates the position of the given @p annotation on the given @p page by a distance @p delta in normalized coordinates.
+         *
+         * Consecutive translations applied to the same @p annotation are merged together on the undo stack if the
+         * BeingMoved flag is set on the @P annotation
+         *
+         * @since 0.17 (KDE 4.11)
+         */
+        void translatePageAnnotation( int page, Annotation *annotation, const Okular::NormalizedPoint & delta );
+
+
+        /**
+         * Edits the plain text contents of the given @p annotation on the given @p page.
+         *
+         * The contents are set to @p newContents with cursor position @p newCursorPos.
+         * The previous cursor position @p prevCursorPos and previous anchor position @p prevAnchorPos
+         * must also be supplied so that they can be restored if the edit action is undone.
+         *
+         * The Annotation's internal contents should not be modified prior to calling this method.
+         *
+         * @since 0.17 (KDE 4.11)
+         */
+        void editPageAnnotationContents( int page, Annotation* annotation, const QString & newContents,
+                                         int newCursorPos, int prevCursorPos, int prevAnchorPos );
 
         /**
          * Tests if the @p annotation can be removed
@@ -426,6 +451,18 @@ class OKULAR_EXPORT Document : public QObject
          */
         void setPageTextSelection( int page, RegularAreaRect * rect, const QColor & color );
 
+        /**
+         * Returns true if there is an undo command available; otherwise returns false.
+         * @since 0.17 (KDE 4.11)
+         */
+        bool canUndo() const;
+
+        /**
+         * Returns true if there is a redo command available; otherwise returns false.
+         * @since 0.17 (KDE 4.11)
+         */
+        bool canRedo() const;
+
         /**
          * Describes the possible search types.
          */
@@ -535,7 +572,7 @@ class OKULAR_EXPORT Document : public QObject
          * Prints the document to the given @p printer.
          */
         bool print( QPrinter &printer );
-        
+
         /**
          * Returns the last print error in case print() failed
          * @since 0.11 (KDE 4.5)
@@ -682,7 +719,6 @@ class OKULAR_EXPORT Document : public QObject
         */
         void setAnnotationEditingEnabled( bool enable );
 
-
     public Q_SLOTS:
         /**
          * This slot is called whenever the user changes the @p rotation of
@@ -701,6 +737,63 @@ class OKULAR_EXPORT Document : public QObject
          */
         void cancelSearch();
 
+        /**
+         * Undo last edit command
+         * @since 0.17 (KDE 4.11)
+         */
+        void undo();
+
+        /**
+         * Redo last undone edit command
+         * @since 0.17 (KDE 4.11)
+         */
+        void redo();
+
+        /**
+         * Edit the text contents of the specified @p form on page @p page to be @p newContents.
+         * The new text cursor position (@p newCursorPos), previous text cursor position (@p prevCursorPos),
+         * and previous cursor anchor position will be restored by the undo / redo commands.
+         * @since 0.17 (KDE 4.11)
+         */
+        void editFormText( int pageNumber,
+                           Okular::FormFieldText* form,
+                           const QString & newContents,
+                           int newCursorPos,
+                           int prevCursorPos,
+                           int prevAnchorPos );
+
+        /**
+         * Edit the selected list entries in @p form on page @p page to be @p newChoices.
+         * @since 0.17 (KDE 4.11)
+         */
+        void editFormList( int pageNumber,
+                           Okular::FormFieldChoice* form,
+                           const QList<int> & newChoices );
+
+
+        /**
+         * Set the active choice in the combo box @p form on page @p page to @p newText
+         * The new cursor position (@p newCursorPos), previous cursor position
+         * (@p prevCursorPos), and previous anchor position (@p prevAnchorPos)
+         * will be restored by the undo / redo commands.
+         *
+         * @since 0.17 (KDE 4.11)
+         */
+        void editFormCombo( int pageNumber,
+                            Okular::FormFieldChoice *form,
+                            const QString & newText,
+                            int newCursorPos,
+                            int prevCursorPos,
+                            int prevAnchorPos );
+
+        /**
+         * Set the states of the group of form buttons @p formButtons on page @p page to @p newButtonStates.
+         * The lists @p formButtons and @p newButtonStates should be the same length and true values
+         * in @p newButtonStates indicate that the corresponding entry in @p formButtons should be enabled.
+         */
+        void editFormButtons( int pageNumber,
+                              const QList< Okular::FormFieldButton* > & formButtons,
+                              const QList< bool > & newButtonStates );
 
     Q_SIGNALS:
         /**
@@ -809,6 +902,18 @@ class OKULAR_EXPORT Document : public QObject
          */
         void processMovieAction( const Okular::MovieAction *action );
 
+        /**
+         * This signal is emmitted whenever the availability of the undo function changes
+         * @since 0.17 (KDE 4.11)
+         */
+        void canUndoChanged( bool undoAvailable );
+
+        /**
+         * This signal is emmitted whenever the availability of the redo function changes
+         * @since 0.17 (KDE 4.11)
+         */
+        void canRedoChanged( bool redoAvailable );
+
         /**
          * This signal is emitted whenever an rendition action is triggered and the UI should process it.
          *
@@ -816,9 +921,56 @@ class OKULAR_EXPORT Document : public QObject
          */
         void processRenditionAction( const Okular::RenditionAction *action );
 
+        /**
+         * This signal is emmitted whenever the contents of the given @p annotation are changed by an undo
+         * or redo action.
+         *
+         * The new contents (@p contents), cursor position (@p cursorPos), and anchor position (@p anchorPos) are
+         * included
+         * @since 0.17 (KDE 4.11)
+         */
+        void annotationContentsChangedByUndoRedo( Okular::Annotation* annotation, const QString & contents, int cursorPos, int anchorPos );
+
+        /**
+         * This signal is emmitted whenever the text contents of the given text @p form on the given @p page
+         * are changed by an undo or redo action.
+         *
+         * The new text contents (@p contents), cursor position (@p cursorPos), and anchor position (@p anchorPos) are
+         * included
+         * @since 0.17 (KDE 4.11)
+         */
+        void formTextChangedByUndoRedo( int page, Okular::FormFieldText* form, const QString & contents, int cursorPos, int anchorPos );
+
+        /**
+         * This signal is emmitted whenever the selected @p choices for the given list @p form on the
+         * given @p page are changed by an undo or redo action.
+         * @since 0.17 (KDE 4.11)
+         */
+        void formListChangedByUndoRedo( int page, Okular::FormFieldChoice* form, const QList< int > & choices );
+
+        /**
+         * This signal is emmitted whenever the active @p text for the given combo @p form on the
+         * given @p page is changed by an undo or redo action.
+         * @since 0.17 (KDE 4.11)
+         */
+        void formComboChangedByUndoRedo( int page, Okular::FormFieldChoice* form, const QString & text, int cursorPos, int anchorPos );
+
+        /**
+         * This signal is emmitted whenever the state of the specified group of form buttons (@p formButtons) on the
+         * given @p page is changed by an undo or redo action.
+         * @since 0.17 (KDE 4.11)
+         */
+        void formButtonsChangedByUndoRedo( int page, const QList< Okular::FormFieldButton* > & formButtons );
     private:
         /// @cond PRIVATE
         friend class DocumentPrivate;
+        friend class Part;
+        friend class ::DocumentItem;
+        friend class EditAnnotationContentsCommand;
+        friend class EditFormTextCommand;
+        friend class EditFormListCommand;
+        friend class EditFormComboCommand;
+        friend class EditFormButtonsCommand;
         /// @endcond
         DocumentPrivate *const d;
 
