@@ -36,8 +36,6 @@ Q_OBJECT
         ThumbnailList(QWidget *parent, Okular::Document *document);
         ~ThumbnailList();
 
-        // inherited: return thumbnails observer id
-        uint observerId() const { return THUMBNAILS_ID; }
         // inherited: create thumbnails ( inherited as a DocumentObserver )
         void notifySetup( const QVector< Okular::Page * > & pages, int setupFlags );
         // inherited: hilihght current thumbnail ( inherited as DocumentObserver )
